<?php
/**
*
* @package phpBB3 SEO Sitemap
* @version $Id: sitemap.php 8987 2013-08-30 18:13:02Z Shredder $
* @copyright (c) 2013 www.phpbb-work.ru
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
* @based on http://js-php.ru/web-development/php-development/phpbb3-simply-sitemap/
*/

define('IN_PHPBB', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : './';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
include($phpbb_root_path . 'common.' . $phpEx);

// Start session management
$user->session_begin();
$auth->acl($user->data);

$f_xml = '';
$url_count = 0;

if (($f_xml = $cache->get("_sitemap_seo_file")) === false)
{
	generate_sitemap();

	if ($config['sitemap_seo_cache'])
	{
		$cache->put("_sitemap_seo_file", $f_xml, 3600*$config['sitemap_seo_cache']);
	}
}

header("Content-type: application/xml");
print $f_xml;


function generate_sitemap()
{
	global $auth, $config, $db, $f_xml, $phpbb_seo, $url_count, $phpbb_root_path, $phpEx;

	$excluded_forums = explode(',', $config['sitemap_seo_excluded']);
	$limit = $config['sitemap_seo_url_limit'];

	$f_begin = '<?xml version="1.0" encoding="UTF-8"?>'."\r\n".'<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">'."\r\n";
	$f_end = '</urlset>';

	// Generate forums list

	$index_count = 1;

	$sql = 'SELECT f.forum_id, f.forum_name, f.forum_topics, f.forum_topics_per_page, MAX(t.topic_time) AS last_topic_time
		FROM ' . FORUMS_TABLE . ' f, ' . TOPICS_TABLE . ' t
		WHERE forum_type = ' . FORUM_POST . '
		AND forum_topics > 0 AND topic_approved = 1 AND t.forum_id = f.forum_id
		GROUP BY f.forum_id';
	$result = $db->sql_query($sql);

	while ($data = $db->sql_fetchrow($result))
	{
		$forum_id = (int) $data['forum_id'];

		if ($auth->acl_get('f_list', $forum_id) && !in_array($forum_id, $excluded_forums))
		{
			$lastpost = date("Y-m-d", $data['last_topic_time']);

			$topics_per_page = ($data['forum_topics_per_page']) ? $data['forum_topics_per_page'] : $config['topics_per_page'];
			$total_pages = intval(($data['forum_topics'] - 1) / $topics_per_page);

			for ($page = 0; $page <= $total_pages; $page++)
			{
				if (!empty($phpbb_seo->seo_opt['url_rewrite']))
				{
					if ($page == 0)
					{
						$phpbb_seo->set_url($data['forum_name'], $forum_id, $phpbb_seo->seo_static['forum']);
						$forum_url = append_sid("{$phpbb_root_path}viewforum.$phpEx", 'f=' . $forum_id);
					}
					else
					{
						$forum_url = append_sid("{$phpbb_root_path}viewforum.$phpEx", 'f=' . $forum_id . "&amp;start=" . ($page * $topics_per_page));
					}

					$forum_url = $phpbb_seo->drop_sid($forum_url);
				}
				else
				{
					if ($page == 0)
					{
						$forum_url = generate_board_url() . "/viewforum.$phpEx?f=$forum_id";
					}
					else
					{
						$forum_url = generate_board_url() . "/viewforum.$phpEx?f=$forum_id&amp;start=" . ($page * $topics_per_page);
					}
				}

				if ($url_count && ($url_count % ($limit * $index_count)) == 0)
				{
					$f_xml = $f_begin . $f_xml . $f_end;

					$w = fopen('sitemap/'.$index_count.'.xml', 'w');
					fwrite($w, $f_xml);
					fclose($w);

					$f_xml = '';
					$index_count += 1;
				}

				add_to_record($forum_url, $lastpost, $config['sitemap_seo_freq_f'], $config['sitemap_seo_prior_f']);
			}
		}
	}

	// Generate topics list

	$add_sql = '';
	if (isset($phpbb_seo->seo_opt['url_rewrite']))
	{
		$add_sql =  ', topic_url';
	}

	$sql = 'SELECT topic_id' . $add_sql . ', topic_type, topic_replies, topic_last_post_time, forum_id
		FROM ' . TOPICS_TABLE . '
		WHERE topic_approved = 1
		AND topic_status <> ' . ITEM_MOVED;
	$result = $db->sql_query($sql);

	while ($data = $db->sql_fetchrow($result))
	{
		$forum_id = (int) $data['forum_id'];
		$topic_id = (int) $data['topic_id'];
		$topic_type = (int) $data['topic_type'];

		if ($topic_type == POST_GLOBAL || ($auth->acl_get('f_list', $forum_id) && !in_array($forum_id, $excluded_forums)))
		{
			$lastpost = date("Y-m-d", $data['topic_last_post_time']);
			$freq = $config['sitemap_seo_freq_' . $topic_type . ''];
			$prior = $config['sitemap_seo_prior_' . $topic_type . ''];

			$total_pages = intval($data['topic_replies'] / $config['posts_per_page']);

			for ($page = 0; $page <= $total_pages; $page++)
			{
				if (!empty($phpbb_seo->seo_opt['url_rewrite']))
				{
					if ($page == 0)
					{
						$phpbb_seo->prepare_iurl($data, 'topic', $data['topic_type'] == POST_GLOBAL ? $phpbb_seo->seo_static['global_announce'] : $phpbb_seo->seo_url['forum'][$forum_id]);
						$view_topic_url = append_sid("{$phpbb_root_path}viewtopic.$phpEx", 'f=' . $forum_id . '&amp;t=' . $topic_id);
					}
					else
					{
						$view_topic_url = append_sid("{$phpbb_root_path}viewtopic.$phpEx", 'f=' . $forum_id . '&amp;t=' . $topic_id . "&amp;start=" . ($page * $config['posts_per_page']));
					}

					$view_topic_url = $phpbb_seo->drop_sid($view_topic_url);
				}
				else
				{
					if ($page == 0)
					{
						$view_topic_url = generate_board_url() . "/viewtopic.$phpEx?" . (($data['topic_type'] == POST_GLOBAL) ? '' : "f=$forum_id&amp;") . "t=$topic_id";
					}
					else
					{
						$view_topic_url = generate_board_url() . "/viewtopic.$phpEx?" . (($data['topic_type'] == POST_GLOBAL) ? '' : "f=$forum_id&amp;") . "t=$topic_id&amp;start=" . ($page * $config['posts_per_page']);
					}
				}

				if (($url_count % ($limit * $index_count)) == 0)
				{
					$f_xml = $f_begin . $f_xml . $f_end;

					$w = fopen('sitemap/'.$index_count.'.xml', 'w');
					fwrite($w, $f_xml);
					fclose($w);

					$f_xml = '';
					$index_count += 1;
				}

				add_to_record($view_topic_url, $lastpost, $freq, $prior);
			}
		}
	}

	if ($f_xml)
	{
		$f_xml = $f_begin . $f_xml . $f_end;
	}

	if ($index_count > 1)
	{
		if ($f_xml)
		{
			$w = fopen('sitemap/'.$index_count.'.xml', 'w');
			fwrite($w, $f_xml);
			fclose($w);
		}

		$f_xml = '<?xml version="1.0" encoding="UTF-8"?>'."\r\n";
		$f_xml .= '<sitemapindex xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">'."\r\n";

		for ($index = 1; $index <= $index_count; $index++)
		{
			$f_xml .= '<sitemap>'."\r\n";
			$f_xml .= '<loc>' . generate_board_url() . '/sitemap/' . $index . '.xml</loc>'."\r\n";
			$f_xml .= '<lastmod>' . date('Y-m-d\TH:i:s+00:00', time()) . '</lastmod>'."\r\n";
			$f_xml .= '</sitemap>'."\r\n";
		}

		$f_xml .= '</sitemapindex>';
	}
}

function add_to_record($loc, $lastpost, $freq, $prior)
{
	global $f_xml, $url_count;

	$f_xml .= '<url>'."\r\n";
	$f_xml .= '<loc>'.$loc.'</loc>'."\r\n";

	if ($lastpost)
	{
		$f_xml .= '<lastmod>'.$lastpost.'</lastmod>'."\r\n";
	}
	if ($freq)
	{
		$f_xml .= '<changefreq>'.$freq.'</changefreq>'."\r\n";
	}
	if ($prior)
	{
		$f_xml .= '<priority>'.$prior.'</priority>'."\r\n";
	}
	$f_xml .= '</url>'."\r\n";

	$url_count += 1;
}

?>